/* Exercise 5. Album catalogue (with extensions) */

#include <stdio.h>
#include <string.h>

#define MAX_LENGTH  50
#define MAX_SONGS   15
#define MAX_ALBUMS  3

struct Song {
    char title[MAX_LENGTH];
    int  length;
};

struct Group {
    char name[MAX_LENGTH];
    char city[MAX_LENGTH];
    int n_members;
};

struct Album {
    char title[MAX_LENGTH];
    int year;
    struct Group group;
    int n_songs;
    struct Song songs[MAX_SONGS];
};

void printAlbum(struct Album a);

int main(void) {
    
    struct Album catalogue[MAX_ALBUMS];   /* Catalog array */
    int i, j;
    int end;
    int nAlbums;	/* Number of albums read from the keyboard */
    int keepReading;
    
    char group_to_search[MAX_LENGTH];
    int year_to_search;
    int found;
    
    /* Read albums */
    keepReading = 1;
    for(i=0; i<MAX_ALBUMS && keepReading; i++) {
        printf("\nEnter album [%i]\n", i);
        
        /* Title */
        printf("Title: ");
        scanf("%[^\n]", catalogue[i].title);
        
        /* Year */
        printf("Year: ");
        scanf("%i", &catalogue[i].year); getchar();	// getchar is added to avoid skipping one value in reading
        
        /* Group */
        printf("Group\n");
        printf("Name: ");
        scanf("%[^\n]", catalogue[i].group.name); getchar();
        printf("City: ");
        scanf("%[^\n]", catalogue[i].group.city);
        printf("# of members: ");
        scanf("%i", &catalogue[i].group.n_members);
        
        /* Songs */
        printf("Songs\n");
        do {
            printf("# of songs (1-%i): ", MAX_SONGS);
            scanf("%i", &catalogue[i].n_songs); getchar();
        } while(catalogue[i].n_songs <= 0 && catalogue[i].n_songs > MAX_SONGS);
        
        for(j=0; j<catalogue[i].n_songs; j++) {
            /* Song j data */
            printf("Song [%i]\n", j);
            printf("Title: ");
            scanf("%[^\n]", catalogue[i].songs[j].title);
            printf("Length: ");
            scanf("%i", &catalogue[i].songs[j].length); getchar();
        }
        
        /* Ask for continue */
        printf("Do you want to continue? (1: yes, 0: no) ");
        scanf("%i", &keepReading); getchar();
    }
    
    nAlbums = i;		/* Store number of read albums! */
    printf("%i albums were read. \n", nAlbums);
    
    /* Extension */
    
    end = 0;
    do {
        int option;
        
        /* Options menu */
        printf("\nCatalogue read. Choose an option: ");
        printf("\n1. Print catalogue");
        printf("\n2. Search by group");
        printf("\n3. Search by year");
        printf("\n4. End\n");
        scanf("%i", &option); getchar();
        
        switch(option) {
            case 1:
                /* Print catalog */
                for(i=0; i<nAlbums; i++) {
                    printf("\nAlbum [%i]\n", i);
                    printAlbum(catalogue[i]);
                }
                break;
                
            case 2:
                /* Search by group name */
                printf("Enter group name: ");
                scanf("%[^\n]", group_to_search);
                printf("Searching for %s \n", group_to_search);
                
                found = 0;
                for(i=0; i<nAlbums; i++) {
                    if( strcmp(catalogue[i].group.name, group_to_search) == 0) {
                        found++;
                        
                        printf("Album [%i]\n", i);
                        printf("Title: %s\n", catalogue[i].title);
                        printf("Year: %i\n", catalogue[i].year);
                        printf("Group: %s (%s) #%i\n",
                               catalogue[i].group.name,
                               catalogue[i].group.city,
                               catalogue[i].group.n_members);
                        
                        printf("Songs (%i)\n",  catalogue[i].n_songs);
                        
                        for(j=0; j<catalogue[i].n_songs; j++) {
                            printf("%i. %s (%i) \n",
                                   j,
                                   catalogue[i].songs[j].title,
                                   catalogue[i].songs[j].length);
                        }
                    }
                }
                
                if(found == 0) {
                    printf("\nGroup not found\n");
                }
                
                break;
                
            case 3:
                /* Search by year */
                printf("Enter year: ");
                scanf("%i", &year_to_search);
                
                found = 0;
                for(i=0; i<nAlbums; i++) {
                    if( catalogue[i].year == year_to_search) {
                        found++;
                        
                        printf("Album [%i]\n", i);
                        printf("Title: %s\n", catalogue[i].title);
                        printf("Year: %i\n", catalogue[i].year);
                        printf("Group: %s (%s) #%i\n",
                               catalogue[i].group.name,
                               catalogue[i].group.city,
                               catalogue[i].group.n_members);
                        
                        printf("Songs (%i)\n",  catalogue[i].n_songs);
                        
                        for(j=0; j<catalogue[i].n_songs; j++) {
                            printf("%i. %s (%i) \n",
                                   j,
                                   catalogue[i].songs[j].title,
                                   catalogue[i].songs[j].length);
                        }
                    }
                }
                
                if(found == 0) {
                    printf("\nYear not found\n");
                }
                
                break;
                
            case 4: /* End program */
                end = 1;
                break;
                
            default:
                printf("\nOption unknown\n");
                break;
        }
    } while(!end);
    
    /* End program */
    return 0;
}

void printAlbum(struct Album a) {	// Functions --> Lesson 7
    int j;
    
    printf("Title: %s\n", a.title);
    printf("Year: %i\n", a.year);
    printf("Group: %s (%s) #%i\n",
           a.group.name,
           a.group.city,
           a.group.n_members);
    
    printf("Songs (%i)\n",  a.n_songs);
    for(j=0; j<a.n_songs; j++) {
        printf("%i. %s (%i) \n",
               j,
               a.songs[j].title,
               a.songs[j].length);
    }
}
